from fastapi import APIRouter
from fastapi.responses import HTMLResponse

router = APIRouter()


@router.get("/", response_class=HTMLResponse)
async def index():
    return """
    <html>
        <head>
            <title>ABOT OpenAPI</title>
            <style type="text/css">
                html,body{font-size: 14px;font-family: "微软雅黑";text-align: center;width: 100%;height: 100%;
                min-height: 100%;border:0;line-height: none;}
                p{border: 0;margin: 0;padding: 0;line-height: none;}
                body{padding:0px; margin:0px ;}
                .container{position:relative;height: auto;min-height: 100%;margin: 0 line-height:18px;}
                .container .header{height: 100px;background: #FFFFFF;}
                .container .push{padding-bottom: 100px;}
                .footer{position:relative;height: 100px;margin-top:-100px;background: #FFFFFF;}
            </style>
        </head>
        <body style="text-align:center;">
            <div class="container">
                <div class="header">
                    <h1>欢迎访问 ABOT</h1>
                </div>
                <div class="content">
                    <strong><h1>Abot是一个智能机器人OpenAPI接口平台！<h1></strong>
                </div>
                <div class="push"></div>
            </div>
        </body>
    </html>
    """



import logging
from fastapi import APIRouter, HTTPException, status
from pydantic import BaseModel

from abotserver.providers.jsst import call_jsst

logger = logging.getLogger("gunicorn.access")

router = APIRouter()


class JSSTData(BaseModel):
    server_url: str
    postfix_url: str
    seq_id: str
    cno: str
    usr: str
    psw: str
    sign_key: str
    p: dict


@router.post("/call-jsst/")
async def call_jsst(data: JSSTData, tag: str = 'UNSET'):
    logger.info('Request Tag: {}'.format(tag))
    try:
        response = call_jsst(server_url=data.server_url,
                             postfix_url=data.postfix_url,
                             seq_id=data.seq_id,
                             cno=data.cno,
                             usr=data.usr,
                             psw=data.psw,
                             sign_key=data.sign_key,
                             p=data.p)
    except Exception as exc:
        logger.error(str(exc))
        raise HTTPException(status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                            detail=str(exc))
    return response



import logging
from fastapi import APIRouter, HTTPException, status
from pydantic import BaseModel

from abotserver.providers.hikvision import acs_register

logger = logging.getLogger("gunicorn.access")

router = APIRouter()


class ACSConnection(BaseModel):
    url: str
    uri: str
    xcakey: str
    secret: str
    payload: dict


@router.post("/acs-register/")
async def register_acs(data: ACSConnection, tag: str = 'UNSET'):
    logger.info('Request Tag: {}'.format(tag))
    secret = data.secret
    if not secret:
        secret = "wZJkGrXo04vrtIiX30vx"
    try:
        response = acs_register(url=data.url,
                                uri=data.uri,
                                xcakey=data.xcakey,
                                secret=secret,
                                payload=data.payload)
    except Exception as exc:
        logger.error(str(exc))
        raise HTTPException(status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                            detail=str(exc))
    if response:
        response = response.json()
    return response


from fastapi import FastAPI
from .routers import index, aliyun, hikvision


app = FastAPI(title="ABOT API Docs", version="v1.0")

app.include_router(index.router, tags=["index"])
app.include_router(aliyun.router, tags=["aliyun"])
app.include_router(hikvision.router, tags=["hikvision"])



import json
from aliyunsdkcore.client import AcsClient
from aliyunsdknlp_automl.request.v20191111 import GetPredictResultRequest


def call_aliyun_nlp(access_key_id, access_key_secret, endpoint, api_version,
                    region_id, model_id, model_version, content):

    client = AcsClient(access_key_id, access_key_secret, region_id)
    request = GetPredictResultRequest.GetPredictResultRequest()

    request.set_ModelId(model_id)
    request.set_ModelVersion(model_version)
    request.set_endpoint(endpoint)
    request.set_version(api_version)
    request.set_Content(content)
    response = client.do_action_with_exception(request)
    return json.loads(response)


import logging
from fastapi import APIRouter, HTTPException, status
from pydantic import BaseModel

from abotserver.providers.aliyun import call_aliyun_nlp

logger = logging.getLogger("gunicorn.access")

router = APIRouter()


class AliyunNLP(BaseModel):
    access_key_id: str
    access_key_secret: str
    endpoint: str
    api_version: str
    region_id: str
    model_id: str
    model_version: str
    content: str


@router.post("/aliyun-nlp-service/")
async def call_aliyun_nlp_service(data: AliyunNLP, tag: str = 'UNSET'):
    try:
        logger.info('Request Tag: {}'.format(tag))
        response = call_aliyun_nlp(access_key_id=data.access_key_id,
                                   access_key_secret=data.access_key_secret,
                                   endpoint=data.endpoint,
                                   api_version=data.api_version,
                                   region_id=data.region_id,
                                   model_id=data.model_id,
                                   model_version=data.model_version,
                                   content=data.content)
    except Exception as exc:
        logger.error(str(exc))
        raise HTTPException(status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                            detail=str(exc))

    return response if response else None




import os
import hashlib
import requests
import json

JIESHUN_DIR = os.path.join(os.environ['HOME'], 'jieshun')
JIESHUN_TOKEN_PATH = os.path.join(JIESHUN_DIR, 'token')


def store_token(token):
    if not os.path.exists(JIESHUN_DIR):
        os.mkdir(JIESHUN_DIR)

    with open(JIESHUN_TOKEN_PATH, mode='w+') as f:
        f.writelines(token)


def get_exist_token():
    if not os.path.exists(JIESHUN_DIR):
        return None

    with open(JIESHUN_TOKEN_PATH, mode='r+') as f:
        return f.read()


def fetch_token(server_url, cno, usr, psw):
    login_url = server_url + "login"
    payload = {
        "cno": cno,
        "usr": usr,
        "psw": psw
    }
    headers = {
        "Content-Type": "application/json",
    }
    req = requests.post(login_url, headers=headers, data=json.dumps(payload)).json()
    return req.get("token", None) if req else None


def call_jsst(server_url, postfix_url, seq_id, cno, usr, psw, sign_key, p):
    token = get_exist_token()
    if not token:
        token = (fetch_token(server_url=server_url, cno=cno, usr=usr, psw=psw))
        store_token(token)
    data = {
        "cno": cno,
        "tn": token,
        "p": p,
        "seqId": seq_id
    }
    url = server_url + postfix_url
    content = json.dumps(data) + sign_key
    sn = hashlib.md5(content.encode('UTF-8')).hexdigest()
    headers = {
        "Content-Type": "application/json",
        "sn": sn,
    }
    try:
        req = requests.post(url, data=json.dumps(data), headers=headers)
        return req.json()
    except Exception:
        token = (fetch_token(server_url=server_url, cno=cno, usr=usr, psw=psw))
        store_token(token)



import json
import datetime
import requests
import hmac
import base64
from hashlib import sha256


def signature_generating(method='POST',
                         content_type="application/json",
                         date=None,
                         xcakey="29836964",
                         secret=None,
                         uri="/artemis/api/visitor/v1/orderless/register"):
    content = "{}\n*/*\n{}\n{}\nx-ca-key:{}\n{}".format(method, content_type, date, xcakey, uri)
    signature = base64.b64encode(hmac.new(secret.encode('utf8'),
                                          content.encode('utf8'),
                                          digestmod=sha256).digest())
    return signature


def acs_register(url, uri, xcakey, payload, secret):
    content_type = "application/json"
    date = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
    headers = {
        "Accept": "*/*",
        "Content-Type": content_type,
        "Date": date,
        "x-Ca-Key": xcakey,
        "X-Ca-Signature": signature_generating(date=date,
                                               secret=secret,
                                               uri=uri),
        "X-Ca-Signature-Headers": "x-ca-key"
    }
    return requests.post(url=url, headers=headers, data=json.dumps(payload), verify=False)



import traceback
import logging

from gunicorn import glogging

logging.basicConfig()

__all__ = ("Logger", )


class Logger(glogging.Logger):

    access_fmt = r"%(asctime)s [%(process)d] [%(levelname)s] %(message)s"

    def access(self, resp, req, environ, request_time):
        """ See http://httpd.apache.org/docs/2.0/logs.html#combined
        for format details
        """

        if not (self.cfg.accesslog or self.cfg.logconfig or
           self.cfg.logconfig_dict or
           (self.cfg.syslog and not self.cfg.disable_redirect_access_to_syslog)):
            return

        # wrap atoms:
        # - make sure atoms will be test case insensitively
        # - if atom doesn't exist replace it by '-'
        safe_atoms = self.atoms_wrapper_class(self.atoms(resp, req, environ,
            request_time))

        try:
            self.access_log.info(self.cfg.access_log_format, safe_atoms)
        except:
            self.error(traceback.format_exc())


class LoggingMixin:
    """Convenience super-class to have a logger configured with the class name"""

    def __init__(self, context=None):
        self._set_context(context)

    @property
    def log(self) -> Logger:
        """Returns a logger."""
        try:
            # FIXME: LoggingMixin should have a default _log field.
            return self._log  # type: ignore
        except AttributeError:
            self._log = logging.getLogger(self.__class__.__module__ + '.' + self.__class__.__name__)
            return self._log

    def _set_context(self, context):
        if context is not None:
            set_context(self.log, context)


def set_context(logger, value):
    """
    Walks the tree of loggers and tries to set the context for each handler

    :param logger: logger
    :param value: value to set
    """
    _logger = logger
    while _logger:
        for handler in _logger.handlers:
            try:
                handler.set_context(value)
            except AttributeError:
                # Not all handlers need to have context passed in so we ignore
                # the error when handlers do not have set_context defined.
                pass
        if _logger.propagate is True:
            _logger = _logger.parent
        else:
            _logger = None





import sys
import signal
import hashlib
import time
import psutil
from time import sleep
from typing import Dict, List, NoReturn
from lockfile.pidlockfile import PIDLockFile

from .logging import LoggingMixin


def check_if_pidfile_process_is_running(pid_file: str, process_name: str):
    """
    Checks if a pidfile already exists and process is still running.
    If process is dead then pidfile is removed.

    :param pid_file: path to the pidfile
    :param process_name: name used in exception if process is up and
        running
    """
    pid_lock_file = PIDLockFile(path=pid_file)
    # If file exists
    if pid_lock_file.is_locked():
        # Read the pid
        pid = pid_lock_file.read_pid()
        if pid is None:
            return
        try:
            # Check if process is still running
            proc = psutil.Process(pid)
            if proc.is_running():
                raise Exception("The {0} is already running under PID {1}.".format(process_name, pid))
        except psutil.NoSuchProcess:
            # If process is dead remove the pidfile
            pid_lock_file.break_lock()


class GunicornMonitor(LoggingMixin):
    """
    Runs forever, monitoring the child processes of @gunicorn_master_proc and
    restarting workers occasionally or when files in the plug-in directory
    has been modified.

    Each iteration of the loop traverses one edge of this state transition
    diagram, where each state (node) represents
    [ num_ready_workers_running / num_workers_running ]. We expect most time to
    be spent in [n / n]. `bs` is the setting webserver.worker_refresh_batch_size.
    The horizontal transition at ? happens after the new worker parses all the
    dags (so it could take a while!)
       V ────────────────────────────────────────────────────────────────────────┐
    [n / n] ──TTIN──> [ [n, n+bs) / n + bs ]  ────?───> [n + bs / n + bs] ──TTOU─┘
       ^                          ^───────────────┘
       │
       │      ┌────────────────v
       └──────┴────── [ [0, n) / n ] <─── start
    We change the number of workers by sending TTIN and TTOU to the gunicorn
    master process, which increases and decreases the number of child workers
    respectively. Gunicorn guarantees that on TTOU workers are terminated
    gracefully and that the oldest worker is terminated.

    :param gunicorn_master_pid: PID for the main Gunicorn process
    :param num_workers_expected: Number of workers to run the Gunicorn web server
    :param master_timeout: Number of seconds the webserver waits before killing gunicorn master that
        doesn't respond
    :param worker_refresh_interval: Number of seconds to wait before refreshing a batch of workers.
    :param worker_refresh_batch_size: Number of workers to refresh at a time. When set to 0, worker
        refresh is disabled. When nonzero, airflow periodically refreshes webserver workers by
        bringing up new ones and killing old ones.
    :param reload_on_plugin_change: If set to True, SMQ will track files in plugins_folder directory.
        When it detects changes, then reload the gunicorn.
    """

    def __init__(
            self,
            gunicorn_master_pid: int,
            num_workers_expected: int,
            master_timeout: int,
            worker_refresh_interval: int,
            worker_refresh_batch_size: int,
            reload_on_plugin_change: bool,
    ):
        super().__init__()
        self.gunicorn_master_proc = psutil.Process(gunicorn_master_pid)
        self.num_workers_expected = num_workers_expected
        self.master_timeout = master_timeout
        self.worker_refresh_interval = worker_refresh_interval
        self.worker_refresh_batch_size = worker_refresh_batch_size
        self.reload_on_plugin_change = reload_on_plugin_change

        self._num_workers_running = 0
        self._num_ready_workers_running = 0
        self._last_refresh_time = time.monotonic() if worker_refresh_interval > 0 else None
        self._restart_on_next_plugin_check = False

    def _generate_plugin_state(self) -> Dict[str, float]:
        """
        Generate dict of filenames and last modification time of all files in PLUGINS_FOLDER
        directory.
        """

        PLUGINS_FOLDER = None

        if not PLUGINS_FOLDER:
            return {}

        ## Can't support plugins currently.
        # all_filenames: List[str] = []
        # for (root, _, filenames) in os.walk(PLUGINS_FOLDER):
        #     all_filenames.extend(os.path.join(root, f) for f in filenames)
        # plugin_state = {f: self._get_file_hash(f) for f in sorted(all_filenames)}
        # return plugin_state

    @staticmethod
    def _get_file_hash(fname: str):
        """Calculate MD5 hash for file"""
        hash_md5 = hashlib.md5()
        with open(fname, "rb") as f:
            for chunk in iter(lambda: f.read(4096), b""):
                hash_md5.update(chunk)
        return hash_md5.hexdigest()

    def _get_num_ready_workers_running(self) -> int:
        """Returns number of ready Gunicorn workers by looking for READY_PREFIX in process name"""
        workers = psutil.Process(self.gunicorn_master_proc.pid).children()

        def ready_prefix_on_cmdline(proc):
            try:
                cmdline = proc.cmdline()
                if len(cmdline) > 0:  # pylint: disable=len-as-condition
                    return "[ready] " in cmdline[0]
            except psutil.NoSuchProcess:
                pass
            return False

        ready_workers = [proc for proc in workers if ready_prefix_on_cmdline(proc)]
        return len(ready_workers)

    def _get_num_workers_running(self) -> int:
        """Returns number of running Gunicorn workers processes"""
        workers = psutil.Process(self.gunicorn_master_proc.pid).children()
        return len(workers)

    def _wait_until_true(self, fn, timeout: int = 0) -> None:
        """Sleeps until fn is true"""
        start_time = time.monotonic()
        while not fn():
            if 0 < timeout <= time.monotonic() - start_time:
                raise Exception("No response from gunicorn master within {0} seconds".format(timeout))
            sleep(0.1)

    def _spawn_new_workers(self, count: int) -> None:
        """
        Send signal to kill the worker.

        :param count: The number of workers to spawn
        """
        excess = 0
        for _ in range(count):
            # TTIN: Increment the number of processes by one
            self.gunicorn_master_proc.send_signal(signal.SIGTTIN)
            excess += 1
            self._wait_until_true(
                lambda: self.num_workers_expected + excess == self._get_num_workers_running(),
                timeout=self.master_timeout,
            )

    def _kill_old_workers(self, count: int) -> None:
        """
        Send signal to kill the worker.

        :param count: The number of workers to kill
        """
        for _ in range(count):
            count -= 1
            # TTOU: Decrement the number of processes by one
            self.gunicorn_master_proc.send_signal(signal.SIGTTOU)
            self._wait_until_true(
                lambda: self.num_workers_expected + count == self._get_num_workers_running(),
                timeout=self.master_timeout,
            )

    def _reload_gunicorn(self) -> None:
        """
        Send signal to reload the gunciron configuration. When gunciorn receive signals, it reload the
        configuration, start the new worker processes with a new configuration and gracefully
        shutdown older workers.
        """
        # HUP: Reload the configuration.
        self.gunicorn_master_proc.send_signal(signal.SIGHUP)
        sleep(1)
        self._wait_until_true(
            lambda: self.num_workers_expected == self._get_num_workers_running(), timeout=self.master_timeout
        )

    def start(self) -> NoReturn:
        """Starts monitoring the webserver."""
        try:  # pylint: disable=too-many-nested-blocks
            self._wait_until_true(
                lambda: self.num_workers_expected == self._get_num_workers_running(),
                timeout=self.master_timeout,
            )
            while True:
                if not self.gunicorn_master_proc.is_running():
                    sys.exit(1)
                self._check_workers()
                # Throttle loop
                sleep(1)

        except (Exception, OSError) as err:
            self.log.error(err)
            self.log.error("Shutting down webserver")
            try:
                self.gunicorn_master_proc.terminate()
                self.gunicorn_master_proc.wait()
            finally:
                sys.exit(1)

    def _check_workers(self) -> None:
        num_workers_running = self._get_num_workers_running()
        num_ready_workers_running = self._get_num_ready_workers_running()

        # Whenever some workers are not ready, wait until all workers are ready
        if num_ready_workers_running < num_workers_running:
            self.log.debug(
                '[%d / %d] Some workers are starting up, waiting...',
                num_ready_workers_running,
                num_workers_running,
            )
            sleep(1)
            return

        # If there are too many workers, then kill a worker gracefully by asking gunicorn to reduce
        # number of workers
        if num_workers_running > self.num_workers_expected:
            excess = min(num_workers_running - self.num_workers_expected, self.worker_refresh_batch_size)
            self.log.debug(
                '[%d / %d] Killing %s workers', num_ready_workers_running, num_workers_running, excess
            )
            self._kill_old_workers(excess)
            return

        # If there are too few workers, start a new worker by asking gunicorn
        # to increase number of workers
        if num_workers_running < self.num_workers_expected:
            self.log.error(
                "[%d / %d] Some workers seem to have died and gunicorn did not restart them as expected",
                num_ready_workers_running,
                num_workers_running,
            )
            sleep(10)
            num_workers_running = self._get_num_workers_running()
            if num_workers_running < self.num_workers_expected:
                new_worker_count = min(
                    num_workers_running - self.worker_refresh_batch_size, self.worker_refresh_batch_size
                )
                self.log.debug(
                    '[%d / %d] Spawning %d workers',
                    num_ready_workers_running,
                    num_workers_running,
                    new_worker_count,
                )
                self._spawn_new_workers(num_workers_running)
            return

        # Now the number of running and expected worker should be equal

        # If workers should be restarted periodically.
        if self.worker_refresh_interval > 0 and self._last_refresh_time:
            # and we refreshed the workers a long time ago, refresh the workers
            last_refresh_diff = time.monotonic() - self._last_refresh_time
            if self.worker_refresh_interval < last_refresh_diff:
                num_new_workers = self.worker_refresh_batch_size
                self.log.debug(
                    '[%d / %d] Starting doing a refresh. Starting %d workers.',
                    num_ready_workers_running,
                    num_workers_running,
                    num_new_workers,
                )
                self._spawn_new_workers(num_new_workers)
                self._last_refresh_time = time.monotonic()
                return

        # if we should check the directory with the plugin,
        if self.reload_on_plugin_change:
            # compare the previous and current contents of the directory
            new_state = self._generate_plugin_state()
            # If changed, wait until its content is fully saved.
            if new_state != self._last_plugin_state:
                self.log.debug(
                    '[%d / %d] Plugins folder changed. The gunicorn will be restarted the next time the '
                    'plugin directory is checked, if there is no change in it.',
                    num_ready_workers_running,
                    num_workers_running,
                )
                self._restart_on_next_plugin_check = True
                self._last_plugin_state = new_state
            elif self._restart_on_next_plugin_check:
                self.log.debug(
                    '[%d / %d] Starts reloading the gunicorn configuration.',
                    num_ready_workers_running,
                    num_workers_running,
                )
                self._restart_on_next_plugin_check = False
                self._last_refresh_time = time.monotonic()
                self._reload_gunicorn()


import os
import signal
import sys
import logging
import textwrap
import subprocess
import click
import psutil
from time import sleep
from contextlib import suppress
from daemon import DaemonContext
from daemon.pidfile import TimeoutPIDLockFile
from lockfile.pidlockfile import read_pid_from_pidfile

from .process import GunicornMonitor, check_if_pidfile_process_is_running

log = logging.getLogger(__name__)

LOG_DIR = "/var/log/abotserver"
if not os.path.exists(LOG_DIR):
    os.mkdir(LOG_DIR)

RUN_DIR = "/var/run/abotserver"
if not os.path.exists(RUN_DIR):
    os.mkdir(RUN_DIR)


def setup_logging(filename):
    """Creates log file handler for daemon process"""
    root = logging.getLogger()
    handler = logging.FileHandler(filename)
    root.addHandler(handler)
    root.setLevel(logging.INFO)

    return handler.stream


@click.group()
def cli():
    pass


@cli.command("run", short_help="run a server")
@click.option("--bind", default="0.0.0.0:8000", show_default=True,
              type=click.STRING, help="The socket to bind.")
@click.option("--daemon/--no-daemon", default=False, show_default=True, help="Daemonize the Gunicorn process.")
@click.option("--workers", default=1, type=click.INT, show_default=True,
              help="The number of worker processes for handling requests.")
@click.option("--worker-class", default="uvicorn.workers.UvicornWorker", show_default=True,
              type=click.STRING, help="The type of workers to use.")
@click.option("--timeout", default=60, type=click.INT, show_default=True,
              help="Workers silent for more than this many seconds are killed and restarted.")
@click.option("--log-level", default="info", show_default=True,
              type=click.Choice(["debug", "info", "warning", "error", "critical"], case_sensitive=False),
              help="The granularity of Error log outputs.")
@click.option("--env", default=None, show_default=True, help="Set environment variable (key=value)")
def run_server(bind, daemon, workers, worker_class, timeout, log_level, env):
    pid_file = os.path.join(RUN_DIR, f'abotserver.pid')
    daemon_file = os.path.join(LOG_DIR, f'daemon.log')
    check_if_pidfile_process_is_running(pid_file=pid_file, process_name="abotserver")
    print(
        textwrap.dedent(
            """\
            Running the Server with:
            Bind: {bind}
            Workers: {workers} {worker_class}
            Timeout: {timeout}
            LogLevel: {log_level}
            =================================================================\
        """.format(
                bind=bind,
                workers=workers,
                worker_class=worker_class,
                timeout=timeout,
                log_level=log_level,
            )
        )
    )

    run_args = [
        "gunicorn",
        "--bind",
        str(bind),
        "--workers",
        str(workers),
        "--worker-class",
        str(worker_class),
        "--timeout",
        str(timeout),
        "--graceful-timeout",
        str(timeout),
        "--log-level",
        str(log_level),
        "--name",
        "abotserver",
        "--pid",
        pid_file,
    ]

    run_args += ["--access-logfile", os.path.join(LOG_DIR, "access.log")]
    run_args += ["--error-logfile", os.path.join(LOG_DIR, "error.log")]
    run_args += ["--logger-class", "abotserver.logging.Logger"]
    run_args += ["--capture-output"]
    if daemon:
        run_args += ["--daemon"]
    else:
        run_args += ["--reload"]

    if env:
        run_args += ["--env", env]

    run_args += ["abotserver.main:app"]

    gunicorn_master_proc = None

    def kill_proc(signum, _):
        log.info("Received signal: %s. Closing gunicorn.", signum)
        gunicorn_master_proc.terminate()
        with suppress(TimeoutError):
            gunicorn_master_proc.wait(timeout=30)
        if gunicorn_master_proc.poll() is not None:
            gunicorn_master_proc.kill()
        sys.exit(0)

    def monitor_gunicorn(gunicorn_master_pid: int):
        # Register signal handlers
        signal.signal(signal.SIGINT, kill_proc)
        signal.signal(signal.SIGTERM, kill_proc)

        # These run forever until SIG{INT, TERM, KILL, ...} signal is sent
        GunicornMonitor(
            gunicorn_master_pid=gunicorn_master_pid,
            num_workers_expected=workers,
            master_timeout=timeout,
            worker_refresh_interval=30,
            worker_refresh_batch_size=1,
            reload_on_plugin_change=False,
        ).start()

    if daemon:
        handle = setup_logging(daemon_file)
        base, ext = os.path.splitext(pid_file)
        stdout = os.path.join(LOG_DIR, f'daemon.out')
        stderr = os.path.join(LOG_DIR, f'daemon.err')
        with open(stdout, 'w+') as stdout, open(stderr, 'w+') as stderr:
            ctx = DaemonContext(
                pidfile=TimeoutPIDLockFile(f"{base}-monitor{ext}", -1),
                files_preserve=[handle],
                stdout=stdout,
                stderr=stderr,
            )
            with ctx:
                subprocess.Popen(run_args, close_fds=True)

                # Reading pid of gunicorn master as it will be different that
                # the one of process spawned above.
                while True:
                    sleep(0.1)
                    gunicorn_master_proc_pid = read_pid_from_pidfile(pid_file)
                    if gunicorn_master_proc_pid:
                        break

                # Run Gunicorn monitor
                gunicorn_master_proc = psutil.Process(gunicorn_master_proc_pid)
                monitor_gunicorn(gunicorn_master_proc.pid)
    else:
        gunicorn_master_proc = subprocess.Popen(run_args, close_fds=True)
        monitor_gunicorn(gunicorn_master_proc.pid)


@cli.command("stop", short_help="Stop the server")
def stop_server():
    pid_file = os.path.join(RUN_DIR, f'abotserver.pid')
    pid = read_pid_from_pidfile(pidfile_path=pid_file)
    if pid:
        click.echo(click.style(">>> Stop the Server[PID:{}] ......".format(pid), fg='yellow'))
        try:
            os.kill(pid, signal.SIGSTOP)
            click.echo(click.style("[SUCCESS]: {}".format("The server already has been stop."), fg='green'))
        except OSError as e:
            click.echo(click.style("[FAILED]: {}".format(str(e)), fg='red'))
        finally:
            sleep(1)
    else:
        click.echo(click.style("[Failed]: {}".format("The server didn't running."), fg='red'))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['abotserver', 'abotserver.providers', 'abotserver.routers']

package_data = \
{'': ['*']}

install_requires = \
['aliyun-python-sdk-core>=2.13.36,<3.0.0',
 'aliyun-python-sdk-nlp-automl>=0.0.9,<0.0.10',
 'fastapi>=0.79.0,<0.80.0',
 'gunicorn>=20.1.0,<21.0.0',
 'lockfile>=0.12.2,<0.13.0',
 'psutil>=5.9.1,<6.0.0',
 'python-daemon>=2.3.0,<3.0.0',
 'requests>=2.28.1,<3.0.0',
 'uvicorn>=0.18.2,<0.19.0']

entry_points = \
{'console_scripts': ['abotserver = abotserver.cmd:cli']}

setup_kwargs = {
    'name': 'abotserver',
    'version': '2022rc2',
    'description': '',
    'long_description': None,
    'author': 'fineartit',
    'author_email': 'fineartit@outlook.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
